@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 """
-Copyright (c) 2013,2014 Fraunhofer FKIE
+Copyright (c) 2013-2015 Fraunhofer FKIE
 
 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
@@ -11,6 +11,9 @@ are met:
  * Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.
+ * Neither the name of the Fraunhofer organization nor the names of its
+   contributors may be used to endorse or promote products derived from
+   this software without specific prior written permission.
 
 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
 IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
@@ -28,9 +31,9 @@ import os
 import re
 from ..linter import ERROR, WARNING, NOTICE
 from ..cmake import argparse as cmake_argparse
-from ..util import word_split, iteritems
+from ..util import word_split, iteritems, is_sorted
 from .manifest import depends as manifest_depends
-
+from functools import partial
 
 def includes(linter):
     def on_init(info):
@@ -79,6 +82,10 @@ def targets(linter):
     def on_final(info):
         if (info.executables or info.libraries) and info.catkin_components and not os.path.normpath("/catkin-includes") in info.build_includes:
             info.report(ERROR, "MISSING_CATKIN_INCLUDE")
+        if os.path.normpath("/catkin-includes") in info.build_includes:
+            for pkg in info.catkin_components:
+                if os.path.normpath("/%s-includes" % pkg) in info.build_includes:
+                    info.report(WARNING, "DUPLICATE_BUILD_INCLUDE", pkg=pkg)
 
     linter.require(includes)
     linter.require(depends)
@@ -94,6 +101,8 @@ def source_files(linter):
     def on_add_executable(info, cmd, args):
         if "IMPORTED" in args: return
         opts, args = cmake_argparse(args, { "WIN32": "-", "MACOSX_BUNDLE": "-", "EXCLUDE_FROM_ALL": "-"})
+        if not is_sorted(args[1:]):
+            info.report(NOTICE, "UNSORTED_LIST", name="of source files")
         for source_file in args[1:]:
             if not source_file: continue
             source_file = info.package_path(source_file)
@@ -103,6 +112,8 @@ def source_files(linter):
     def on_add_library(info, cmd, args):
         if "IMPORTED" in args: return
         opts, args = cmake_argparse(args, { "STATIC": "-", "SHARED": "-", "MODULE": "-", "EXCLUDE_FROM_ALL": "-"})
+        if not is_sorted(args[1:]):
+            info.report(NOTICE, "UNSORTED_LIST", name="of source files")
         for source_file in args[1:]:
             if not source_file: continue
             source_file = info.package_path(source_file)
@@ -128,6 +139,7 @@ def link_directories(linter):
 def depends(linter):
     def on_init(info):
         info.required_packages = set([])
+        info.test_packages = set([])
         info.catkin_components = set([])
         info.checked_packages = set([])
     def on_find_package(info, cmd, args):
@@ -137,17 +149,28 @@ def depends(linter):
         if args[0] in info.find_packages:
             info.report(ERROR, "DUPLICATE_FIND", pkg=args[0])
         if opts["REQUIRED"]: info.required_packages.add(args[0])
+        if info.condition_is_true("CATKIN_ENABLE_TESTING"):
+            info.test_packages.add(args[0])
+        else:
+            if args[0] in info.test_dep - info.build_dep - info.buildtool_dep:
+                info.report(ERROR, "UNGUARDED_TEST_DEPEND", pkg=args[0])
         if args[0] != "catkin": return
         if "catkin_package" in info.commands:
             info.report(ERROR, "ORDER_VIOLATION", first_cmd="catkin_package", second_cmd=cmd)
         if not opts["REQUIRED"]:
             info.report(WARNING, "MISSING_REQUIRED", pkg="catkin")
             info.required_packages.add("catkin")
+        if not is_sorted(opts["COMPONENTS"]):
+            info.report(NOTICE, "UNSORTED_LIST", name="COMPONENTS")
         for pkg in opts["COMPONENTS"]:
+            info.var["%s_INCLUDE_DIRS" % pkg] = "/%s-includes" % pkg
+            info.var["%s_LIBRARIES" % pkg] = "/%s-libs/library.so" % pkg
             if pkg in info.find_packages:
                 info.report(ERROR, "DUPLICATE_FIND", pkg=pkg)
-            if not info.env.is_catkin_pkg(pkg):
-                info.report(ERROR, "NO_CATKIN_COMPONENT", pkg=pkg)
+            if not info.env.is_known_pkg(pkg):
+                if info.env.ok: info.report(ERROR, "UNKNOWN_PACKAGE", pkg=pkg)
+            elif not info.env.is_catkin_pkg(pkg):
+                if info.env.ok: info.report(ERROR, "NO_CATKIN_COMPONENT", pkg=pkg)
         for pkg in args[1:]:
             if info.env.is_known_pkg(pkg):
                 info.report(ERROR, "MISSING_COMPONENTS", pkg=pkg)
@@ -159,13 +182,13 @@ def depends(linter):
             if arg.endswith("_FOUND"):
                 info.checked_packages.add(arg[0:-6])
     def on_final(info):
-        for pkg in info.required_packages - info.build_dep - info.buildtool_dep:
+        for pkg in info.required_packages - info.build_dep - info.buildtool_dep - info.test_dep:
             if info.env.is_known_pkg(pkg):
                 info.report(ERROR, "MISSING_DEPEND", pkg=pkg, type="build")
         for pkg in info.find_packages - info.required_packages - info.checked_packages:
             if info.env.is_catkin_pkg(pkg):
                 info.report(ERROR, "MISSING_REQUIRED", pkg=pkg)
-        for pkg in info.build_dep - info.find_packages:
+        for pkg in info.build_dep - (info.find_packages - info.test_packages):
             if info.env.is_catkin_pkg(pkg):
                 info.report(ERROR, "UNCONFIGURED_BUILD_DEPEND", pkg=pkg)
 
@@ -176,6 +199,22 @@ def depends(linter):
     linter.add_final_hook(on_final)
 
 
+def tests(linter):
+    def on_test_cmd(info, cmd, args, dep=None):
+        if not info.condition_is_true("CATKIN_ENABLE_TESTING"):
+            info.report(ERROR, "UNGUARDED_TEST_CMD", cmd=cmd)
+        if dep is None: return
+        if not dep in info.test_dep | info.build_dep:
+            info.report(ERROR, "MISSING_DEPEND", type="test", pkg=dep)
+
+    linter.require(manifest_depends)
+    linter.add_command_hook("catkin_download_test_data", on_test_cmd)
+    linter.add_command_hook("catkin_add_gtest", partial(on_test_cmd, dep="rosunit"))
+    linter.add_command_hook("catkin_add_nosetests", on_test_cmd)
+    linter.add_command_hook("add_rostest", partial(on_test_cmd, dep="rostest"))
+    linter.add_command_hook("add_rostest_gtest", partial(on_test_cmd, dep="rostest"))
+
+
 def exports(linter):
     def on_init(info):
         info.export_packages = set([])
@@ -184,12 +223,17 @@ def exports(linter):
         info.export_targets = set([])
     def on_catkin_package(info, cmd, args):
         opts, args = cmake_argparse(args, { "INCLUDE_DIRS": "*", "LIBRARIES": "*", "DEPENDS": "*", "CATKIN_DEPENDS": "*", "CFG_EXTRAS": "*", "EXPORTED_TARGETS": "*" })
+        for list_name in ["CATKIN_DEPENDS","DEPENDS","CFG_EXTRAS","EXPORTED_TARGETS"]:
+            if not is_sorted(opts[list_name]):
+                info.report(NOTICE, "UNSORTED_LIST", name=list_name)
         for pkg in opts["CATKIN_DEPENDS"]:
-            if not info.env.is_catkin_pkg(pkg):
-                info.report(ERROR, "SYSTEM_AS_CATKIN_DEPEND", pkg=pkg)
+            if not info.env.is_known_pkg(pkg):
+                if info.env.ok: info.report(ERROR, "UNKNOWN_PACKAGE", pkg=pkg)
+            elif not info.env.is_catkin_pkg(pkg):
+                if info.env.ok: info.report(ERROR, "SYSTEM_AS_CATKIN_DEPEND", pkg=pkg)
         for pkg in opts["DEPENDS"]:
             if info.env.is_catkin_pkg(pkg):
-                info.report(ERROR, "CATKIN_AS_SYSTEM_DEPEND", pkg=pkg)
+                if info.env.ok: info.report(ERROR, "CATKIN_AS_SYSTEM_DEPEND", pkg=pkg)
             elif pkg in info.pkg_modules:
                 info.report(ERROR, "EXPORTED_PKG_CONFIG", pkg=pkg)
             elif not pkg in info.find_packages and not ("%s_INCLUDE_DIRS" % pkg in info.var and "%s_LIBRARIES" % pkg in info.var):
@@ -204,7 +248,12 @@ def exports(linter):
         info.export_targets |= set(opts["EXPORTED_TARGETS"])
     def on_final(info):
         for pkg in info.export_packages - info.export_dep:
-            info.report(ERROR, "MISSING_DEPEND", pkg=pkg, type="run" if info.manifest.package_format < 2 else "build_export")
+            if info.env.is_known_pkg(pkg):
+                if pkg == "message_runtime":
+                    if not pkg in info.exec_dep:
+                        info.report(ERROR, "MISSING_DEPEND", pkg=pkg, type="run" if info.manifest.package_format < 2 else "exec")
+                else:
+                    info.report(ERROR, "MISSING_DEPEND", pkg=pkg, type="run" if info.manifest.package_format < 2 else "build_export")
         for pkg in (info.find_packages & info.build_dep & info.export_dep) - info.export_packages:
             if re.search(r"_(msg|message)s?(_|$)", pkg) and info.env.is_catkin_pkg(pkg):
                 info.report (WARNING, "SUGGEST_CATKIN_DEPEND", pkg=pkg)
@@ -277,7 +326,7 @@ def installs(linter):
         info.install_includes = False
         info.install_files = set([])
     def on_install(info, cmd, args):
-        install_type = ""
+        install_type = None
         opts, args =  cmake_argparse(args, { "PROGRAMS" : "*", "FILES": "*", "TARGETS": "*", "DIRECTORY" : "?", "DESTINATION" : "?", "ARCHIVE DESTINATION": "?", "LIBRARY DESTINATION": "?", "RUNTIME DESTINATION": "?" })
         if opts["PROGRAMS"]:
             install_type = "PROGRAMS"
@@ -289,6 +338,10 @@ def installs(linter):
         if opts["TARGETS"]:
             install_type = "TARGETS"
             info.install_targets |= set(opts["TARGETS"])
+        if install_type is None:
+            return
+        if install_type != "DIRECTORY" and not is_sorted(opts[install_type]):
+            info.report(NOTICE, "UNSORTED_LIST", name=install_type)
         for dest in [ "DESTINATION", "ARCHIVE DESTINATION", "LIBRARY DESTINATION", "RUNTIME DESTINATION" ]:
             if opts[dest] is None: continue
             if not info.is_catkin_target(opts[dest]):
@@ -356,6 +409,9 @@ def message_generation(linter):
             info.report(ERROR, "CATKIN_ORDER_VIOLATION", cmd=cmd)
         if "catkin_package" in info.commands:
             info.report(ERROR, "ORDER_VIOLATION", first_cmd="catkin_package", second_cmd=cmd)
+        opts, args = cmake_argparse(args, { "DIRECTORY": "?", "FILES": "*" })
+        if not is_sorted(opts["FILES"]):
+            info.report(NOTICE, "UNSORTED_LIST", name="FILES")
     def on_generate_msg(info, cmd, args):
         info.uses_catkin = True
         if info.manifest.is_metapackage():
@@ -365,6 +421,8 @@ def message_generation(linter):
         if "catkin_package" in info.commands:
             info.report(ERROR, "ORDER_VIOLATION", first_cmd="catkin_package", second_cmd=cmd)
         opts, args = cmake_argparse(args, { "DEPENDENCIES": "*" })
+        if not is_sorted(opts["DEPENDENCIES"]):
+            info.report(NOTICE, "UNSORTED_LIST", name="DEPENDENCIES")
         info.msg_dep |= set(opts["DEPENDENCIES"])
     def on_final(info):
         if info.manifest.is_metapackage(): return
@@ -402,6 +460,7 @@ def all(linter):
     linter.require(source_files)
     linter.require(link_directories)
     linter.require(depends)
+    linter.require(tests)
     linter.require(exports)
     linter.require(name_check)
     linter.require(pkg_config)
