@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 """
-Copyright (c) 2013,2014 Fraunhofer FKIE
+Copyright (c) 2013-2015 Fraunhofer FKIE
 
 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
@@ -11,6 +11,9 @@ are met:
  * Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.
+ * Neither the name of the Fraunhofer organization nor the names of its
+   contributors may be used to endorse or promote products derived from
+   this software without specific prior written permission.
 
 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
 IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
@@ -29,10 +32,9 @@ import sys
 from functools import total_ordering
 from fnmatch import fnmatch
 from copy import copy
-from .packages import find_packages
 from .cmake import ParserContext, argparse as cmake_argparse, SyntaxError as CMakeSyntaxError
 from .diagnostics import msg
-from .util import iteritems
+from .environment import CatkinEnvironment
 
 ERROR = 0
 WARNING = 1
@@ -73,6 +75,7 @@ class LintInfo(object):
         self.targets = set([])
         self.executables = set([])
         self.libraries = set([])
+        self.conditionals = []
         self.var = {}
         self.parent_var = {}
         self.messages = []
@@ -113,6 +116,23 @@ class LintInfo(object):
         if subdir is not None: catkin_dir = os.path.join(catkin_dir, subdir)
         return os.path.normpath(path).startswith(os.path.normpath(catkin_dir))
 
+    def condition_is_true(self, expr):
+        ret = False
+        for c in self.conditionals:
+            if c.expr == expr and c.value == False: return False
+            if c.expr == expr and c.value == True: ret = True
+        return ret
+
+    def condition_is_false(self, expr):
+        return not self.condition_is_true(expr)
+
+
+class IfCondition(object):
+    def __init__(self, expr, value):
+        self.expr = expr
+        self.value = value
+
+
 class CMakeLinter(object):
     def __init__(self, env):
         self.env = env
@@ -149,9 +169,8 @@ class CMakeLinter(object):
         self._final_hooks.append(cb)
 
     def _read_file(self, filename):
-        f = open(filename, "r")
-        content = f.read()
-        f.close()
+        with open(filename, "r") as f:
+            content = f.read()
         return content
 
     def _include_file(self, info, args):
@@ -267,6 +286,16 @@ class CMakeLinter(object):
         if pragma == "ignore":
             info.ignore_messages |= set([ a.upper() for a in args ])
 
+    def _handle_if(self, info, cmd, args):
+        if cmd == "if":
+            info.conditionals.append(IfCondition(" ".join(args), True))
+        if cmd == "else":
+            if len(info.conditionals) > 0:
+                info.conditionals[-1].value = False
+        if cmd == "endif":
+            if len(info.conditionals) > 0:
+                info.conditionals.pop()
+
     def _parse_file(self, info, filename):
         save_file = info.file
         save_line = info.line
@@ -276,7 +305,11 @@ class CMakeLinter(object):
             info.file = cur_file
             info.line = 0
             content = self._read_file(filename)
-            for cmd, args, fname, line in self._ctx.parse(content, var=info.var, filename=cur_file):
+            # cur_col is a len(call_stack) list of indentation lists
+            # i.e. each function/macro invocation has its own indentation list
+            cur_col = [[None]]
+            cur_depth = 0
+            for cmd, args, (fname, line, column) in self._ctx.parse(content, var=info.var, filename=cur_file):
                 info.file = fname
                 info.line = line
                 if cmd == "#catkin_lint":
@@ -287,6 +320,34 @@ class CMakeLinter(object):
                 if cmd != cmd.lower():
                     info.report(NOTICE, "CMD_CASE", cmd=cmd)
                     cmd = cmd.lower()
+                depth = self._ctx.call_depth()
+                if depth > cur_depth:
+                    cur_col += [[None]] * (depth-cur_depth)
+                    cur_depth = depth
+                if depth < cur_depth:
+                    del cur_col[depth+1:]
+                    cur_depth = depth
+                if cmd == "else":
+                    if len(cur_col[-1]) < 2: raise CMakeSyntaxError("%s(%d): else() without if()" % (info.file, info.line))
+                    if column != cur_col[-1][-2]:
+                        info.report(NOTICE, "INDENTATION")
+                    cur_col[-1][-1] = None
+                elif cmd in ["endif", "endforeach"]:
+                    cur_col[-1].pop(-1)
+                    if len(cur_col[-1]) == 0: raise CMakeSyntaxError("%s(%d): %s() without %s()" % (info.file, info.line, cmd, cmd[3:]))
+                    if column != cur_col[-1][-1]:
+                        info.report(NOTICE, "INDENTATION")
+                else:
+                    if cur_col[-1][-1] is None:
+                        cur_col[-1][-1] = column
+                        if len(cur_col[-1]) >= 2 and cur_col[-1][-2] >= cur_col[-1][-1]:
+                            info.report(NOTICE, "INDENTATION")
+                    if column != cur_col[-1][-1]:
+                        info.report(NOTICE, "INDENTATION")
+                if cmd in ["if", "foreach"]:
+                    cur_col[-1].append(None)
+                if cmd in ["if", "else", "endif"]:
+                    self._handle_if(info, cmd, args)
                 if cmd == "project":
                     info.var["PROJECT_NAME"] = args[0]
                     info.var["PROJECT_SOURCE_DIR"] = info.var["CMAKE_CURRENT_SOURCE_DIR"]
@@ -342,6 +403,7 @@ class CMakeLinter(object):
         info.ignore_messages = self.ignore_messages
         info.path = path
         info.manifest = manifest
+        info.conditionals = []
         info.var = {
           "CMAKE_CURRENT_SOURCE_DIR" : "/pkg-source",
           "CMAKE_CURRENT_BINARY_DIR" : "/pkg-build",
@@ -370,60 +432,3 @@ class CMakeLinter(object):
             info.report(ERROR, "OS_ERROR", msg=str(err))
         self.messages += info.messages
         self.ignored_messages += info.ignored_messages
-
-class CatkinEnvironment(object):
-    def __init__(self, rosdep_view=None):
-        self.manifests = {}
-        self.known_catkin_pkgs = set([])
-        self.known_other_pkgs = set([])
-        if rosdep_view is None:
-            try:
-                from rosdep2.lookup import RosdepLookup
-                from rosdep2.rospkg_loader import DEFAULT_VIEW_KEY
-                from rosdep2.sources_list import SourcesListLoader
-                sources_loader = SourcesListLoader.create_default()
-                lookup = RosdepLookup.create_from_rospkg(sources_loader=sources_loader)
-                self.rosdep_view = lookup.get_rosdep_view(DEFAULT_VIEW_KEY)
-            except Exception as err:
-                sys.stderr.write("catkin_lint: cannot load rosdep database: %s\n" % str(err))
-                sys.stderr.write("catkin_lint: unknown dependencies will be ignored\n")
-                self.rosdep_view = {}
-        else:
-            self.rosdep_view = rosdep_view
-        self.cache = {}
-
-    def add_path(self, path):
-        if not os.path.isdir(path):
-            return []
-        realpath = os.path.realpath(path)
-        if realpath in self.cache:
-            return self.cache[realpath]
-        pkgs = find_packages(path)
-        found = []
-        for p, m in iteritems(pkgs):
-            is_catkin = True
-            for e in m.exports:
-                if e.tagname == "build_type" and e.content != "catkin":
-                    is_catkin = False
-                    break
-            if is_catkin:
-                self.known_catkin_pkgs.add(m.name)
-                pm = ( os.path.join(path, p), m )
-                self.manifests[m.name] = pm
-                found.append(pm)
-            else:
-                self.known_other_pkgs.add(m.name)
-        self.cache[realpath] = found
-        return found
-
-    def is_catkin_pkg(self, name):
-        return name in self.known_catkin_pkgs
-
-    def is_system_pkg(self, name):
-        return name in self.rosdep_view.keys() or name in self.known_other_pkgs
-
-    def is_known_pkg(self, name):
-        return self.is_catkin_pkg(name) or self.is_system_pkg(name)
-
-    def has_rosdep(self):
-        return len(self.rosdep_view.keys()) > 0
