@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 """
-Copyright (c) 2013,2014 Fraunhofer FKIE
+Copyright (c) 2013-2015 Fraunhofer FKIE
 
 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
@@ -11,6 +11,9 @@ are met:
  * Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.
+ * Neither the name of the Fraunhofer organization nor the names of its
+   contributors may be used to endorse or promote products derived from
+   this software without specific prior written permission.
 
 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
 IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
@@ -64,6 +67,12 @@ def write_atomic(filepath, data):
             os.unlink(filepath_tmp)
 
 
+def is_sorted(lst, key=lambda x, y: x < y):
+    for i, el in enumerate(lst[1:]):
+        if key(el, lst[i-1]):
+            return False
+    return True
+
 # Python 3 compatibility without sacrificing the speed gain of iteritems in Python 2
 try:
     iteritems = dict.iteritems
