@@ -1,6 +1,6 @@
 import unittest
 import catkin_lint.checks.build as cc
-from .helper import create_env, create_manifest, mock_lint
+from .helper import create_env, create_manifest, create_manifest2, mock_lint
 
 import sys
 sys.stderr = sys.stdout
@@ -98,6 +98,15 @@ class ChecksBuildTest(unittest.TestCase):
         checks=cc.depends)
         self.assertEqual([ "DUPLICATE_FIND" ], result)
 
+        result = mock_lint(env, pkg,
+            """
+            project(mock)
+            find_package(catkin REQUIRED COMPONENTS other_catkin)
+            find_package(unknown_package REQUIRED)
+            """,
+        checks=cc.depends)
+        self.assertEqual([], result)
+
         result = mock_lint(env, pkg,
             """
             project(mock)
@@ -116,6 +125,15 @@ class ChecksBuildTest(unittest.TestCase):
         checks=cc.depends)
         self.assertEqual([ "NO_CATKIN_COMPONENT", "MISSING_DEPEND" ], result)
 
+        result = mock_lint(env, pkg,
+            """
+            project(mock)
+            find_package(catkin REQUIRED COMPONENTS unknown_package)
+            find_package(other_catkin REQUIRED)
+            """,
+        checks=cc.depends)
+        self.assertEqual([ "UNKNOWN_PACKAGE" ], result)
+
         result = mock_lint(env, pkg,
             """
             project(mock)
@@ -143,6 +161,56 @@ class ChecksBuildTest(unittest.TestCase):
             """,
         checks=cc.depends)
         self.assertEqual([], result)
+        result = mock_lint(env, pkg,
+            """
+            project(mock)
+            find_package(catkin REQUIRED)
+            if(CATKIN_ENABLE_TESTING)
+            find_package(other_catkin REQUIRED)
+            endif()
+            """,
+        checks=cc.depends)
+        self.assertEqual(["UNCONFIGURED_BUILD_DEPEND"], result)
+
+        pkg = create_manifest("mock", test_depends=[ "other_catkin" ])
+        result = mock_lint(env, pkg,
+            """
+            project(mock)
+            find_package(catkin REQUIRED)
+            if(CATKIN_ENABLE_TESTING)
+            find_package(other_catkin REQUIRED)
+            endif()
+            """,
+        checks=cc.depends)
+        self.assertEqual([], result)
+        result = mock_lint(env, pkg,
+            """
+            project(mock)
+            find_package(catkin REQUIRED)
+            find_package(other_catkin REQUIRED)
+            """,
+        checks=cc.depends)
+        self.assertEqual(["UNGUARDED_TEST_DEPEND"], result)
+        pkg = create_manifest("mock", build_depends=[ "other_catkin"], test_depends=[ "other_catkin" ])
+        result = mock_lint(env, pkg,
+            """
+            project(mock)
+            find_package(catkin REQUIRED)
+            if(CATKIN_ENABLE_TESTING)
+            find_package(other_catkin REQUIRED)
+            endif()
+            """,
+        checks=cc.depends)
+        self.assertEqual(["UNCONFIGURED_BUILD_DEPEND"], result)
+        result = mock_lint(env, pkg,
+            """
+            project(mock)
+            find_package(catkin REQUIRED)
+            find_package(other_catkin REQUIRED)
+            """,
+        checks=cc.depends)
+        self.assertEqual([], result)
+
 
     @patch("os.path.isfile", lambda x: x == os.path.normpath("/mock-path/src/mock.cpp"))
     def do_targets(self):
@@ -172,6 +240,18 @@ class ChecksBuildTest(unittest.TestCase):
         checks=cc.targets)
         self.assertEqual([ "MISSING_CATKIN_INCLUDE" ], result)
 
+        result = mock_lint(env, pkg,
+            """
+            project(mock)
+            find_package(catkin REQUIRED COMPONENTS other_catkin)
+            catkin_package()
+            include_directories(${catkin_INCLUDE_DIRS} ${other_catkin_INCLUDE_DIRS})
+            add_executable(mock_prog src/mock.cpp)
+            target_link_libraries(mock_prog ${catkin_LIBRARIES})
+            """,
+        checks=cc.targets)
+        self.assertEqual([ "DUPLICATE_BUILD_INCLUDE" ], result)
+
         result = mock_lint(env, pkg,
             """
             project(mock)
@@ -322,6 +402,29 @@ class ChecksBuildTest(unittest.TestCase):
         self.assertEqual([ "UNINSTALLED_DEPEND" ], result)
 
 
+    def test_tests(self):
+        env = create_env()
+        pkg = create_manifest("mock")
+        result = mock_lint(env, pkg,
+            """
+            project(mock)
+            find_package(catkin REQUIRED)
+            catkin_download_test_data()
+            """,
+        checks=cc.tests)
+        self.assertEqual(["UNGUARDED_TEST_CMD"], result)
+        result = mock_lint(env, pkg,
+            """
+            project(mock)
+            find_package(catkin REQUIRED)
+            if(CATKIN_ENABLE_TESTING)
+            add_rostest()
+            endif()
+            """,
+        checks=cc.tests)
+        self.assertEqual(["MISSING_DEPEND"], result)
+
+
     @patch("os.path.isfile", lambda x: x == os.path.normpath("/mock-path/src/mock.cpp"))
     @patch("os.path.isdir", lambda x: x in [ os.path.normpath("/mock-path/include"), os.path.normpath("/mock-path/include/mock") ])
     def do_exports(self):
@@ -369,7 +472,7 @@ class ChecksBuildTest(unittest.TestCase):
             find_library(Stuff_LIBRARIES stuff)
             catkin_package(
             CATKIN_DEPENDS other_catkin
-            DEPENDS other_system Stuff
+            DEPENDS Stuff other_system
             )
             """,
         checks=cc.exports)
@@ -416,6 +519,19 @@ class ChecksBuildTest(unittest.TestCase):
         checks=cc.exports)
         self.assertEqual([ "SYSTEM_AS_CATKIN_DEPEND" ], result)
 
+        result = mock_lint(env, pkg,
+            """
+            project(mock)
+            find_package(catkin REQUIRED)
+            find_package(other_catkin REQUIRED)
+            find_package(unknown_package REQUIRED)
+            catkin_package(
+            CATKIN_DEPENDS other_catkin unknown_package
+            )
+            """,
+        checks=cc.exports)
+        self.assertEqual([ "UNKNOWN_PACKAGE" ], result)
+
         result = mock_lint(env, pkg,
             """
             project(mock)
@@ -713,6 +829,38 @@ class ChecksBuildTest(unittest.TestCase):
         checks=cc.message_generation)
         self.assertEqual([ "MISSING_DEPEND", "UNCONFIGURED_MSG_DEPEND", "MISSING_MSG_DEPEND", "MISSING_MSG_DEPEND" ], result)
 
+    def test_format2_message_exports(self):
+        env = create_env()
+        pkg = create_manifest2("mock", build_depends=["message_generation"], exec_depends=["message_runtime"])
+        result = mock_lint(env, pkg,
+            """
+            project(mock)
+            find_package(catkin REQUIRED COMPONENTS message_generation)
+            catkin_package(CATKIN_DEPENDS message_runtime)
+            """,
+        checks=cc.exports)
+        self.assertEqual([], result)
+
+        pkg = create_manifest2("mock", build_depends=["message_generation"], build_export_depends=["message_runtime"])
+        result = mock_lint(env, pkg,
+            """
+            project(mock)
+            find_package(catkin REQUIRED COMPONENTS message_generation)
+            catkin_package(CATKIN_DEPENDS message_runtime)
+            """,
+        checks=cc.exports)
+        self.assertEqual([], result)
+
+        pkg = create_manifest2("mock", build_depends=["message_generation"])
+        result = mock_lint(env, pkg,
+            """
+            project(mock)
+            find_package(catkin REQUIRED COMPONENTS message_generation)
+            catkin_package(CATKIN_DEPENDS message_runtime)
+            """,
+        checks=cc.exports)
+        self.assertEqual(["MISSING_DEPEND"], result)
+
 
     @patch("os.path", posixpath)
     def test_posix(self):
